#EqualLogic

####Table of Contents

1. [Overview](#overview)
2. [Requirements](#Requirements)
3. [Usage](#Usage)
4. [Volume Create](#VolumeCreate)
5. [Volume Clone](#VolumeClone)
6. [Get Volume IQN](#GetVolumeIQN)
7. [Create CHAP User](#CreateCHAPUser)
8. [Access Volume Using IQN](#AccessVolumeUsingIQN)
9. [Access Volume Using CHAP](#AccessVolumeUsingCHAP)

##Overview

The EqualLogic storage device module is designed to extend the support for managing the EqualLogic storage configuration, using Puppet and its Network Device functionality.
The EqualLogic storage device module has been written and tested against the EqualLogic Device (firmware version 7.0.1). 

Note: This module may be compatible with other versions.

## Requirements

As a Puppet agent cannot be directly installed on the EqualLogic storage, it can either be managed from the Puppet Master server or through an intermediate proxy system running a puppet agent. 
The requirements for the proxy system are:

 * Puppet 2.7.+
 * EqualLogic Python API's
 * Python-paramiko-1.7.5-2.1.el6.noarch
 * We will also need following python SNMP packages
	1.	pysnmp-4.2.5
	2.	pyasn1-0.1.7

### EqualLogic Python API

The EqualLogic Python API is a collection of EqualLogic RA modules.

## Usage

### Device Setup

To configure an EqualLogic storage device, the device *type* must be `equallogic`.
The device can either be configured within */etc/puppet/device.conf*, or, preferably, create an individual config file for each device within a sub-folder.
This allows you to run the Puppet against individual devices, rather than all configured devices.

To run the Puppet against a single device, you can run the following command:

    puppet device --deviceconfig /etc/puppet/device/[device].conf

Example configuration `/etc/puppet/device/equallogic1.example.com.conf`:

    [equallogic1.example.com]
      type equallogic
      url https://root:secret@equallogic1.example.com

##Volume Create

###Description

The EqualLogic type/provider supports the functionality to create and delete the volumes 
on the EqualLogic storage.

####Summary of Properties

1. state : This parameter sets status for the volume. 
		The possible values are: "online" or "offline".
	
2. multihostaccess : This parameter enables or disables multi-host access on a volume.
		The possible values are: "enable" or "disable".
        
####Summary of Parameters

1. name : This parameter defines the name of the volume.

2. size : This parameter defines the size of the volume.

3. ensure : (Mandatory) This parameter is required to call the Create or Delete method.
    	The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls the delete method.

4. poolname : This parameter defines the pool name where volume would be created. The default value is 'default'.   

5. thinprovision : This parameter enables thin provisioning on this volume.
		The possible values are: "enable" or "disable".
    
6. snapresv : This parameter defines the amount of space, as a percentage of the volume size, to reserve for snapshot.
    
7. thinminreserve : This parameter sets the minimum reserved size for thin provisioned volume configured as percentage of total volume size. 
		This value cannot be less than 10%, for example: 25%.
    
8. thingrowthwarn : This parameter sets the warning threshold percentage for thin-provisioned volume. 
		When thin-reserve reaches this value, a warning message is displayed. The default value is 60%.
    
9. thingrowthmax : This parameter sets the maximum growth percentage for thin volume. 
		When thin-reserve reaches this value, the volume is set to offline. The default value is 80%.
    
10. thinwarnsoftthres : This parameter specifies whether or not a thin provisioning sends an initiator warning when passing the in-use warning threshold.

11. thinwarnhardthres : This parameter specifies whether or not a thin provisioning allows the volume to remain online after reaching the max-growth threshold.


        
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and supported files:

   	vol_create.pp
	
Sample init.pp file:
   
	class equallogic {
		equallogic::vol_create { 'demoVolume':
			ensure				=>  present,
            size				=> '1m',
			state				=> 'online',
			poolname            => 'default',
			multihostaccess		=> 'disable',
			thinprovision		=> 'enable',
			snapresv			=> '20%',
			thinminreserve		=> '20%',
			thingrowthwarn		=> '30%',
			thingrowthmax		=> '40%',
			thinwarnsoftthres	=> '50%',
			thinwarnhardthres	=> '60%',
		}
	}

You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device
    
    
##Volume Clone

###Description

The clone method clones the existing volume as per the parameters specified in the definition.
        
####Summary of Parameters

1. name : This parameter defines the name of the newly cloned volume.

2. volume_name : This parameter defines the name of the volume, which will be cloned.

3. ensure : (Mandatory) This parameter is required to call the Create or Destroy method.
		The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls the delete method.
    
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and supported files:

   	vol_clone.pp
	
Sample init.pp file:
   
	class equallogic {
		equallogic::vol_clone { 'demoCloneVolume':
			ensure              =>  present,
			volume_name   		=> 'demoVolume',
		}
	}

You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device
    
    
##Get Volume IQN

###Description

This method displays the available volume's IQN as per the parameters specified in the definition.
        
####Summary of Parameters

1. name : This parameter defines the name of the volume.

2. ensure : (Mandatory) This parameter is required to call the Create or Destroy method.
		The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls the Delete method.
    
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and supported files:

   	vol_get_iqn.pp
	
Sample init.pp file:
   
	class equallogic {
		equallogic::vol_get_iqn { 'demoVolume':
			ensure       	=> present,
		}
	}

You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device
    	
##Create CHAP User

###Description

The chap type/provider supports the functionality to create and delete the chap user on the EqualLogic storage.
        
####Summary of Parameters

1. name : This parameter specifies the CHAP username (63 alphanumeric characters or fewer) to which access is limited.

2. ensure : (Mandatory) This parameter is required to call the Create or Destroy method.
		The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls the Delete method.
        
3. chap_password : This parameter specifies the CHAP password (254 characters or fewer). If the password is not specified, then it is autogenerated.

4. state : This parameter sets the status for the CHAP account.
		The possible values are: "online" or "offline".
    
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and the supported files:

   	vol_create_chap_user.pp
	
Sample init.pp file:
   
	class equallogic {
		equallogic::vol_create_chap_user { 'demoChap:
			ensure              =>  present
			chap_password   	=> 'mychap123',
			state 				=> 'enable'
		}
	}
	
You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device

##Access Volume Using IQN

###Description

This method is used to create or remove the access for the IP addresses or IQN numbers in the existing volume in an EqualLogic storage device. It creates an accessID for that IP address or IQN number in that volume.
        
####Summary of Parameters

1. name : This parameter defines the name of the volume.

2. ensure : (Mandatory) This parameter is required to call the Create or Destroy method.
		The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls the Delete method.
        
3. iqnorip : This parameter defines the iSCSI initiator name to which the access is limited (up to 222 characters, an asterisk can be used in place of any character or sub-string) or IP address to which the access is limited (in dotted quad notation).
    
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and supported files:

   	vol_access_iqn.pp
	
Sample init.pp file:	
   
	class equallogic {
		equallogic::vol_access_iqn { 'demoVolume':
			ensure              =>  present,
			iqnorip       		=> 'iqn.1992-01.example.com'
		}
	}

You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device

##Access Volume Using CHAP

###Description


This method creates or removes the access on EqualLogic volume for a CHAP account. It creates or removes access on the volume, for the CHAP user, available on the EqualLogic.
        
####Summary of Parameters

1. name : This parameter defines the name of the volume.

2. ensure : (Mandatory) This parameter is required to call the Create or Destroy method.
		The possible values are: "present" or "absent".
    	If ensure parameter is set to "present", the RA module calls the Create method.
    	If ensure parameter is set to "absent", the RA module calls delete method.
        
3. chap_user_name : This parameter specifies the list of CHAP user names (63 alphanumeric characters or fewer) to which access is limited.
    
####Usage

Refer to the examples in the manifest directory. The following file contains the details for the sample init.pp and supported files:

   	vol_access_chap.pp
   
	Sample init.pp file:
   
	class equallogic {
		equallogic::vol_access_chap { 'demoVolume':
			ensure              =>  present,
			chap_user_name 		=> 'demoChap'
		}
	}


You can create an init.pp file based on the above sample files and run the "puppet device" command , for example:

	puppet device



