#!/opt/puppet/bin/ruby

require 'uri'
require 'net/https'
require 'trollop'
require 'pathname'
require 'timeout'
require 'fileutils'

opts = Trollop::options do
  opt :server, 'equallogic server address', :type => :string, :required => true
  opt :port, 'equallogic server port', :default => 443
  opt :username, 'equallogic server username', :type => :string, :required => true
  opt :password, 'equallogic server password', :type => :string, :default => ENV['PASSWORD']
  opt :timeout, 'command timeout', :default => 240
  opt :community_string, 'equallogic community string', :default => 'public'
end
MIB_DIR = '/var/lib/net-snmp/mib_indexes'
response = ''

if !opts[:password]
  puts 'No password defined'
  exit 1
end

begin
  Timeout.timeout(opts[:timeout]) do
    # intentionally doing this to let it finish
    folder = Pathname.new(__FILE__).parent
    fact_retrieve_command = "#{folder}/equallogic_facts.rb --server #{opts[:server]} --port #{opts[:port]} --username '#{opts[:username]}' "\
      "--community #{opts[:community_string]}"
    fact_retrieve_command << " --password '#{opts[:password]}'" if opts[:password]
    response = `#{fact_retrieve_command}`
  end
rescue Timeout::Error, Exception => e
  response = "#{e.message}\n#{e.backtrace}"
  exit 1
ensure
  eqlogic_json = File.join('/opt/Dell/ASM/cache', "#{opts[:server]}.json")
  if File.exists? eqlogic_json
    puts File.read(eqlogic_json)
    exit 0
  else
    puts response
    exit 1
  end
end
