#!/opt/puppet/bin/ruby

require 'trollop'
require 'json'
require 'pathname'
require 'puppet'

@opts = Trollop::options do
  opt :server, 'equallogic server address', :type => :string, :required => true
  opt :port, 'equallogic server port', :default => 443
  opt :username, 'equallogic server username', :type => :string, :required => true
  opt :password, 'equallogic server password', :type => :string, :default => ENV['PASSWORD']
  opt :timeout, 'command timeout', :default => 180
  opt :community, 'equallogic community string', :default => 'public'
end

file_path = File.join(Pathname.new(__FILE__).parent.parent,'files')
@python_file_path = File.join(file_path,'pythonEquallogic.py')
@member_file_path = File.join(file_path,'member')
@pool_file_path = File.join(file_path,'pool')
@python_snmp_path = File.join(file_path,'EquallogicSNMP.py')
ENV['PS_GROUP_ADDR'] = @opts[:server]
ENV['PS_GROUP_USER'] = @opts[:username]
ENV['PS_GROUP_PASSWORD'] = @opts[:password]
ENV['LD_LIBRARY_PATH'] = "#{ENV['LD_LIBRARY_PATH']}:#{Pathname.new(__FILE__).parent.parent}/files/lib"

def connect

end

def get_volumes
  vol_names = '1.3.6.1.4.1.12740.5.1.7.1.1.4'
  vol_props = '1.3.6.1.4.1.12740.5.1.7.1.1'
  volume_columns = [
      "Index",
      "RowStatus",
      "PsvId",
      "Name",
      "Site",
      "Description",
      "AccessType",
      "Size",
      "AdminStatus",
      "ReservedPercentage",
      "SnapWarningLevel",
      "SnapDeletionPolicy",
      "AutoloadBalancing",
      "TargetAlias",
      "TargetIscsiName",
      "NodeIndex",
      "HdrDigests",
      "DataDigests",
      "CloneSrcPsvId",
      "ReplService",
      "MultInitiator",
      "StoragePoolIndex",
      "StoragePoolSourceIndex",
      "TargetLocalMemberId",
      "TargetIndex",
      "ThinProvision",
      "MinThinReserve",
      "ThinWarnPercentage",
      "ThinMaxGrowPercentage",
      "ShrinkAutoAdjust",
      "ReplRollbackSeqNum",
      "ThinWarnMode",
      "Flags",
      "Template",
      "ThinClone",
      "ThinCloneParentMemberId",
      "ThinCloneParentVolIndex",
      "ThinCloneParentPsvId",
      "AdminAccountKey",
      "SCSIQErr"
  ]
  get_resources(@opts[:server], @opts[:community], vol_names, vol_props, volume_columns)
end

def get_volume_statuses
  vol_names = '1.3.6.1.4.1.12740.5.1.7.7.1.1'
  vol_props = '1.3.6.1.4.1.12740.5.1.7.7.1'
  volume_columns = [
      "StatusIndex",
      "StatusPsvId",
      "StatusReservedSpace",
      "StatusReservedSpaceAvail",
      "StatusActualMemberCount",
      "StatusNumSnapshots",
      "StatusCreationTime",
      "StatusAvailable",
      "StatusOperStatus",
      "StatusConnections",
      "StatusLostRaidBlocksAction",
      "StatusNumReplicas",
      "StatusReplReserveSpace",
      "StatusAllocatedSpace",
      "StatusReplResvInUse",
      "StatusReplTxData",
      "StatusNumOnlineSnaps",
      "StatusPagesSharedWithParent",
      "StatusExternalConnections"
  ]
  get_resources(@opts[:server], @opts[:community], vol_names, vol_props, volume_columns)
end

def get_volume_list
  {:volumes => get_volumes, :volume_statuses => get_volume_statuses}
end

def get_resources(host, community, names_mib, volumes_mib, columns)
  walk_string = "snmpwalk -v 2c -O vqe -c #{community} #{host} #{names_mib}"
  Puppet.debug("Executing #{walk_string}")
  volume_count_response = `snmpwalk -v 2c -O vqe -c #{community} #{host} #{names_mib}`
  if $?.to_i == 0
    volume_count = volume_count_response.split("\n").count
  else
    Puppet.debug(volume_count_response)
    raise Puppet::Error, volume_count_response
  end
  Puppet.debug("Executing: snmpwalk -v 2c -O vqe -c #{community} #{host} #{volumes_mib}")
  volumes_response = `snmpwalk -v 2c -O vqe -c #{community} #{host} #{volumes_mib}`
  if $?.to_i == 0
    volumes = volumes_response
  else
    Puppet.debug(volumes_response)
    raise Puppet::Error, volumes_response
  end
  p_index = 0
  p_list = []
  p_list[0] = []
  volumes.split("\n").each_with_index do |line,i|
    if (i != 0) && ((i % volume_count) == 0)
      p_index += 1
      p_list[p_index] = []
    end
    p_list[p_index].push(line)
  end
  v_list = []
  v_hash_list = []
  p_list.each_with_index do |property,i|
    property.each_with_index do |v_property,j|
      v_list[j] = [] if i == 0
      v_list[j].push(v_property.gsub /"/, '')
      v_hash_list[j] = {} if i == 0
      column_name = columns[v_list[j].count]
      v_hash_list[j][column_name] = v_property.gsub(/"/, '') unless column_name.nil?
    end
  end
  v_hash_list
end

def get_volume_properties(volume_list)
  volume_map = {}
  volume_count_map = {}
  snapshot_properties_map = {}
  volume_counter = 0
  online_volume_counter = 0
  in_use_volume_counter = 0
  iSCSI_conn_counter = 0
  snapshot_counter = 0
  online_snapshot_counter = 0
  in_use_snapshot_counter = 0
  # We are excluding recovery volumes, vss-control and pe-control-vol
  # [XXX] Still do not know the purpose of pe-control-vol, as it not inside the EQL, UI or referenced on Google
  vol_exclude_re = /^(.*(-\d{4}-\d\d-\d\d-(\d\d:){2}\d\d\.\d+\.\d+)|vss-control|pe-control-vol)$/
  volume_list[:volumes].each_with_index do |volume,i|
    next if volume['Name'].match(vol_exclude_re)
    vol = map_volume_properties(volume,volume_list[:volume_statuses][i])
    volume_counter += 1
    online_volume_counter += 1 if vol['Volume Status'] == 'online'
    in_use_volume_counter += 1 if vol['Volume Use'] != '0'
    snapshot_counter += vol['Number of Snapshot'].to_i
    iSCSI_conn_counter += vol['ISCSI Connections'].to_i
    online_snapshot_counter += vol['Number of Online Snapshots'].to_i
    in_use_snapshot_counter += vol['Snapshots in Use'].to_i
    volume_map[volume['Name']] = JSON.pretty_generate(vol)
  end
  volume_count_map['Total Volumes'] = "#{volume_counter}"
  volume_count_map['Online'] = "#{online_volume_counter}"
  volume_count_map['ISCSI Connections'] = "#{iSCSI_conn_counter}"
  volume_count_map['In Use'] = "#{in_use_volume_counter}"
  snapshot_properties_map['Online'] = "#{online_snapshot_counter}"
  snapshot_properties_map['In Use'] = "#{in_use_snapshot_counter}"
  {
      'VolumesProperties' => JSON.pretty_generate(volume_map),
      'VolumesInfo' => JSON.pretty_generate(volume_count_map),
      'Volumes' => "#{volume_counter}",
      'Snapshots' => "#{snapshot_counter}",
      'SnapshotsInfo' => JSON.pretty_generate(snapshot_properties_map)
  }
end

def map_volume_properties (volume, volume_status)
  volumePropertiesMap = {}
  volumePropertiesMap['Storage Pool'] = volume['StoragePoolIndex'] == "1" ? "default" : "Pool #{volume['StoragePoolIndex']}"
  volumePropertiesMap['Reported Size'] = (volume['Size'].to_f / 1024).to_s + "GB"
  volumePropertiesMap['Volume Reserve'] = (volume_status['StatusReservedSpaceAvail'].to_f / 1024).to_s + "GB"
  volumePropertiesMap['Snapshot Reserve'] = volume['ReservedPercentage'] + "%"
  volumePropertiesMap['Borrowed Space'] = volume['StatusReplReserveSpace'].to_i.to_s + "MB"
  volumePropertiesMap['Volume Status'] = volume['RowStatus'] == "1" ? "online" : "not online"
  volumePropertiesMap['Volume Use'] = volume_status['StatusConnections']
  volumePropertiesMap['Replication Partner'] = "" #to-verify
  volumePropertiesMap['SyncRep Status'] = "" #to-verify
  volumePropertiesMap['Number of Snapshot'] = volume_status['StatusNumSnapshots']
  volumePropertiesMap['Number of Online Snapshots'] = volume_status['StatusNumOnlineSnaps']
  volumePropertiesMap['ISCSI Connections'] = volume_status['StatusExternalConnections']
  volumePropertiesMap['Snapshots in Use'] = volume_status['StatusReplResvInUse']
  volumePropertiesMap['TargetIscsiName'] = volume['TargetIscsiName']
  volumePropertiesMap
end

def get_general_setting_info
  general_map = {}
  response = get_grp_param_response
  if (response.match("% Error"))
    raise Puppet::Error, "Failed to fetch the equallogic group parameters: #{response}"
  end
  response.split("\n").each do |line|
    res = line.split(":")
    name = res[0].to_s.strip
    value = res[1].to_s.strip
    if name == "Name"
      general_map['Group Name'] = value
      @data['Group Name'] = value
    elsif name == "Group-Ipaddress"
      general_map['IP Address'] = value
    elsif name == "Management-Ipaddress"
      @data['Management IP'] = value
    elsif name == "Location"
      general_map['Location'] = value
    end
  end
  JSON.pretty_generate(general_map)
end

def get_grp_param_response
  response = `python "#{@python_file_path}" #{@opts[:server]} #{@opts[:username]} #{@opts[:password]} discoverygrpparamsshow`
  response
end

def get_storage_pool_info
  poolMap = {}
  resp = `"#{@pool_file_path}" show`
  if resp.match("% Error")
    raise Puppet::Error, "Failed to fetch the equallogic pools: #{resp}"
  end
  counter = 0
  resp.split("\n").each do |line|
    if counter < 3
      counter = counter + 1
      next
    else
      res = line.split(' ')
      poolMap[res[0].to_s.strip] = {
          'Members' => res[2].to_s.strip,
          'Total' => res[4].to_s.strip}
    end
  end
  JSON.pretty_generate(poolMap)
end

def get_member_count_info
  member_counter = 0
  member_map = {}
  member_status_map = {}
  member_version_map = {}
  resp = `"#{@member_file_path}" show`
  if resp.match("% Error")
    raise Puppet::Error, "Failed to fetch the equallogic pools: #{resp}"
  end
  member_model_list = Array.new
  resp.split("\n").each do |line|
    member = line.to_s.strip
    if member.size > 0
      member_counter += 1
      response = `python "#{@python_file_path}" #{@opts[:server]} #{@opts[:username]} #{@opts[:password]} membershowfordiscovery "#{member}"`
      if response.match("% Error")
        raise Puppet::Error, "Failed to fetch the equallogic member properties: #{response}"
      end
      response.split("\n").each do |ln|
        res = ln.split(":")
        name = res[0].to_s.strip
        value = res[1].to_s.strip
        case name
          when 'Product Family'
            member_map[member] = value
          when 'Status'
            member_status_map[member] = value
          when 'Version'
            member_version_map[member] = value
          else
        end
      end

    end
  end
  @data['Member Model'] = JSON.pretty_generate(member_map)
  @data['Status'] = JSON.pretty_generate(member_status_map)
  @data['Version'] = JSON.pretty_generate(member_version_map)
  member_counter
end

def get_collection_info
  collection_map = {}
  counter = 0
  volume_collection_counter = 0
  snap_collection_counter = 0
  response =  `python "#{@python_file_path}" #{@opts[:server]} #{@opts[:username]} #{@opts[:password]} volumecollectioninfoshow`
  response.split("\n").each do |line|
    if counter < 2
      counter += 1
      next
    elsif line.to_s.strip.length > 0 && line =~ /^\S+/
      volume_collection_counter += 1
    end
  end
  counter = 0
  response = `python "#{@python_file_path}" #{@opts[:server]} #{@opts[:username]} #{@opts[:password]} snapcollectioninfoshow`
  response.split("\n").each do |line|
    if counter < 2
      counter += 1
      next
    elsif line.to_s.strip.length > 0 && line =~ /^\S+/
      snap_collection_counter += 1
    end
  end
  collection_map['Volume Collections'] = "#{volume_collection_counter}"
  collection_map['Custom Snapshot Collections'] = "#{snap_collection_counter}"
  collection_map['Snapshot Collections'] = "#{@data['Snapshots']}"
  JSON.pretty_generate(collection_map)
end

def get_snmp_info(cmd)
  response = `python "#{@python_snmp_path}" "#{@opts[:server]}" "#{@opts[:community]}" #{cmd}`
  if response =~ /Failed|No SNMP/
    'N/A'
  else
    response.to_s.strip
  end
end

def get_group_disk_space
  disk_space_map = {}
  disk_space_map['Snapshot Reserve'] = get_snmp_info('getsnapshotreserve')
  disk_space_map['Replication Reserve'] = get_snmp_info('getreplicationreserve')
  disk_space_map['Delegated'] = get_snmp_info('getdelegated')
  disk_space_map['Free'] = get_snmp_info('getfreespace')
  disk_space_map['Volume Reserve'] = get_snmp_info('getvolumereserve')
  JSON.pretty_generate(disk_space_map)
end

def get_iqn_info
  volume_cmd = "#{Pathname.new(__FILE__).parent.parent}/files/volume show"
  volumes_reponse = `#{volume_cmd}`
  vol_data_hash = {}
  volumes_reponse.split("\n").each_with_index do |line,i|
    # skip first three lines
    next if i <= 2 || line.length == 0
    line.split("\t").each do |vol_data|
      vol_name = vol_data[0].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"] ||= {}
      vol_data_hash["#{vol_name}"]['TargetIscsiName'] = vol_data[1].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['Size'] = vol_data[2].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['Snapshots'] = vol_data[3].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['Status'] = vol_data[4].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['Permission'] = vol_data[5].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['TP'] = vol_data[6].gsub(/\s+/, "")
      vol_data_hash["#{vol_name}"]['Template'] = vol_data[7].gsub(/\s+/, "")
    end
  end
  JSON.pretty_generate(vol_data_hash)
end


def facts
  volume_props_results = get_volume_properties(get_volume_list)
  @data = Hash.new

  {
      'General Settings' => get_general_setting_info,
      'Storage Pools' => get_storage_pool_info,
      'Group Members' => get_member_count_info,
      'Collections' => get_collection_info,
      'Model' => 'EqualLogic',
      'Group Disk Space' => get_group_disk_space,
      'Free Group Space' => get_snmp_info('getfreespace'),
      'Volume_IQN_Information' => get_iqn_info
  }.each do |fact_name,res|
    @data[fact_name] = res
  end

  volume_props_results.each do |k,v|
    @data[k] = v
  end
  @data
end

begin
  results = facts.to_json
  p results
  exit 0
rescue Exception => e
  puts e.message
  puts e.backtrace
  exit 1
ensure
  results ||= {}
  equallogic_cache = '/opt/Dell/ASM/cache'
  Dir.mkdir(equallogic_cache) unless Dir.exists? equallogic_cache
  file_path = File.join(equallogic_cache, "#{@opts[:server]}.json")
  File.write(file_path, results) unless results.empty?
end