# --------------------------------------------------------------------------
# Access Mechanism 
# --------------------------------------------------------------------------

The EqualLogic storage module uses the EqualLogic Python and PSAPIs to interact with the EqualLogic storage device.

# --------------------------------------------------------------------------
#  Supported Functionality
# --------------------------------------------------------------------------

	- Create CHAP user
	- Destroy CHAP user

# -------------------------------------------------------------------------
# Functionality Description
# -------------------------------------------------------------------------


  1. Create

     This method creates the CHAP user as per the parameters specified in the definition. 
   
  2. Destroy

     This method deletes the CHAP user from the storage device.  

# -------------------------------------------------------------------------
# Summary of parameters
# -------------------------------------------------------------------------

    name - This parameter specifies the CHAP username (63 alphanumeric characters or fewer) to which access is limited.

	ensure - (Required) This parameter is required to call the create or destroy method.
			  The possible values are: "present" or "absent".
    If the ensure parameter is set to "present", the module calls the create method.
    If the ensure parameter is set to "absent", the modules calls the destroy method.
	
	chap_password - This parameter specifies the CHAP password (254 characters or fewer). If not specified then password is autogenerated.
	state - This parameter sets the status for the CHAP account {enable | disable}.


# -------------------------------------------------------------------------
# Parameter signature 
# -------------------------------------------------------------------------

#Provide transport and CHAP user properties

	define equallogic::vol_create_chap_user (
		$ensure        = 'present',
		$chap_password   = 'mychap123',
		$state = 'enable'
	) {

		equallogic_volume_chap_user { "${name}":
			ensure        => $ensure,
			chap_password   => $chap_password,
			state => $state,
		}
	}

# --------------------------------------------------------------------------
# Usage
# --------------------------------------------------------------------------
   Refer to the examples in the manifest directory.
   The following file contains the details for the sample init.pp and supported files:

    - vol_create_chap_user.pp
	
   Sample init.pp file:
   
   class equallogic {
		equallogic::vol_create_chap_user { 'demoOne':
			ensure              =>  present
			chap_password   	=> 'mychap123',
			state 				=> 'enable'
		}
	}
	
   You can create an init.pp file based on the above sample file and run the "puppet device" command , for example: 
   # puppet device

#-------------------------------------------------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------------------------------------------------	
