# --------------------------------------------------------------------------
# Access Mechanism 
# --------------------------------------------------------------------------

The EqualLogic storage module uses the EqualLogic Python API to interact with the EqualLogic storage device.

# --------------------------------------------------------------------------
#  Supported Functionality
# --------------------------------------------------------------------------

	- Create Volume Access - CHAP
	- Remove Volume Access - CHAP
	
# -------------------------------------------------------------------------
# Functionality Description
# -------------------------------------------------------------------------

	1. Create Volume Access - CHAP
	
		This method creates access on a volume for a CHAP account. It creates access on the volume, for the CHAP user, available on the EqualLogic.

	2. Remove Volume Access - CHAP
		
		This method deletes the access on a volume for a CHAP account. It deletes the access on the volume, for the CHAP user, available on the EqualLogic.
		
		
# -------------------------------------------------------------------------
# Summary of parameters
# -------------------------------------------------------------------------
	name - This parameter defines the name of the volume.
	
	ensure - (Required) This parameter is required to call the create or destroy method.
            The possible values are: "present" or "absent".
    If ensure parameter is set to "present", the module calls the create method.
    If ensure parameter is set to "absent", the module calls the destroy method.
	
	chap_user_name - This parameter specifies the list of CHAP user names (63 alphanumeric characters or fewer) to which access is limited.
	
# -------------------------------------------------------------------------
# Parameter signature 
# -------------------------------------------------------------------------

#Provide transport and volume properties

	define equallogic::vol_access_chap (
		$ensure,
		$chap_user_name
	) {
		equallogic_volume_access_chap { "${name}":
			ensure => $ensure,
			chap_user_name => $chap_user_name,
		}
	}	
	
# --------------------------------------------------------------------------
# Usage
# --------------------------------------------------------------------------
   Refer to the examples in the manifest directory.
   The following file contains the details for the sample init.pp and supported files:

    - vol_access_chap.pp
   
   Sample init.pp file:
   
   class equallogic {
		equallogic::vol_access_chap { 'demoOne':
			ensure              =>  present,
			chap_user_name 		=> 'mychap'
		}
	}
	
  You can create an init.pp file based on the above sample files and run the "puppet device" command , for example: 
   # puppet device

#-------------------------------------------------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------------------------------------------------	
