# --------------------------------------------------------------------------
# Access Mechanism 
# --------------------------------------------------------------------------

The EqualLogic storage module uses the EqualLogic Python API to interact with the EqualLogic storage device.

# --------------------------------------------------------------------------
#  Supported Functionality
# --------------------------------------------------------------------------

	- Create Volume Access - IQN
	- Remove Volume Access - IQN or IP

# -------------------------------------------------------------------------
# Functionality Description
# -------------------------------------------------------------------------

	1. Create Volume Access - IQN or IP Address
	
		This method is used to create access for the IP addresses or IQN numbers in the existing volume in an EqualLogic storage device.
		It creates an accessID for that IP address or IQN number in that volume.
		
	2. Remove Volume Access - IQN or IP Address

			This method is used to delete access of an IP address or IQN number in the existing volume, where access is already created for that IP address or IQN number in an EqualLogic storage device.

# -------------------------------------------------------------------------
# Summary of parameters
# -------------------------------------------------------------------------
	name - This parameter defines the name of the volume.
	
	ensure - (Required) This parameter is required to call the create or destroy method.
    The possible values are: "present" or "absent".
    If the ensure parameter is set to "present", the module calls the create method.
    If the ensure parameter is set to "absent", the module calls the destroy method.
	
	iqnorip -  This parameter defines the iSCSI initiator name to which access is limited (up to 222 characters, can use an asterisk in place of any character or sub-string) or IP address to which access is limited (in dotted quad notation).
   
# -------------------------------------------------------------------------
# Parameter signature 
# -------------------------------------------------------------------------

#Provide transport and volume properties

	define equallogic::vol_access_iqn (
		$ensure,
		$iqnorip
	) {
		equallogic_volume_access_iqn { "${name}":
			ensure => $ensure,
			iqnorip => $iqnorip,
		}
	}

# --------------------------------------------------------------------------
# Usage
# --------------------------------------------------------------------------
   Refer to the examples in the manifest directory.
   The following file contains the details for the sample init.pp and supported files:

    - vol_access_iqn.pp
	
   Sample init.pp file:	
   
   class equallogic {
		equallogic::vol_access_iqn { 'demoAnup1':
			ensure              =>  present,
			iqnorip       		=> 'iqn.1992-01.example.com'
		}
	}
	
  You can create an init.pp file based on the above sample files and run the "puppet device" command , for example: 
   # puppet device

#-------------------------------------------------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------------------------------------------------	
