# --------------------------------------------------------------------------
# Access Mechanism 
# --------------------------------------------------------------------------

The EqualLogic storage module uses the EqualLogic Python and PSAPIs to interact with the EqualLogic storage device.

# --------------------------------------------------------------------------
#  Supported Functionality
# --------------------------------------------------------------------------

	- Create Volume
	- Destroy Volume

# -------------------------------------------------------------------------
# Functionality Description
# -------------------------------------------------------------------------
  1. Create

     This method creates a volume on EqualLogic storage as per the parameters specified in the definition. 

  2. Destroy

     This method deletes the volume from EqualLogic storage.  


# -------------------------------------------------------------------------
# Summary of parameters
# -------------------------------------------------------------------------

	name - This parameter defines the name of the volume.
	
    ensure - (Required)This parameter is required to call create or delete method.
    The possible values are: "present" or "absent".
    If ensure parameter is set to "present", RA module will call create method.
    If ensure parameter is set to "absent", RA module will call delete method.

    size: This parameter defines the size of the volume.
	
    state: This parameter sets status for the volume. 
		   The possible values are: "online" or "offline".
		   
    poolname: This parameter specifies the pool name where volume would be created. 
               This is an optional parameter. If not specified then volume would be created in 'default' pool
	
    multihostaccess: This parameter enables or disables multi-host access on a volume.
                     The possible values are: "enable" or "disable".
    
	thinprovision: This parameter enables thin provisioning on this volume.
                   The possible values are: "enable" or "disable".
    
	snapresv: This parameter defines the amount of space, as a percentage of the volume size, to reserve for snapshot.
    
	thinminreserve: This parameter sets the minimum reserved size for thin provisioned volume configured as percentage of total volume size. 
	                This value cannot be less than 10%. Example: 25%.
    
	thingrowthwarn: This parameter sets the warning threshold percentage for thin-provisioned volume. 
	                When thin-reserve reaches this value, a warning message is displayed. Default is 60%.
    
	thingrowthmax: This parameter sets the maximum growth percentage for thin volume. 
	               When thin-reserve reaches this value, volume is set to offline. Default is 80%.
    
	thinwarnsoftthres: This parameter specifies whether or not thin provisioning will send an initiator warning when passing the in-use warning threshold.
    
	thinwarnhardthres: This parameter specifies whether or not thin provisioning will allow the volume to remain online after reaching the max-growth threshold.

# -------------------------------------------------------------------------
# Parameter signature 
# -------------------------------------------------------------------------

#Provide transport and create or destroy volume properties

	define equallogic::vol_create (
		$size = 1m,
		$ensure = 'present',
		$snapresv = '20%',
		$state = 'online',
		$multihostaccess = 'disable',
		$thinprovision = 'enable',
		$thinminreserve = '20%',
		$thingrowthwarn = '30%',
		$thingrowthmax = '40%',
		$thinwarnsoftthres = '50%',
		$thinwarnhardthres = '60%'
	) {
		equallogic_volume { "${name}":
			ensure            => $ensure,
			initsize          => $size,
			state	      => $state,
			multihostaccess   => $multihostaccess,
			snapreserve       => $snapresv,
			thinprovision     => $thinprovision,
			thinminreserve    => $thinminreserve,
			thingrowthwarn    => $thingrowthwarn,
			thingrowthmax     => $thingrowthmax,
			thinwarnsoftthres => $thinwarnsoftthres,
			thinwarnhardthres => $thinwarnhardthres,  
		}
	}

# --------------------------------------------------------------------------
# Usage
# --------------------------------------------------------------------------
   Refer to the examples in the manifest directory.
   The following file contains the details for the sample init.pp and supported files:

    - vol_create_destroy.pp
	
   Sample init.pp file:
   
   class equallogic {
		equallogic::vol_create { 'demoAnup200':
			ensure              =>  present,
			size                => '1m',
			state 				=> 'online',
			multihostaccess 	=> 'disable',
			thinprovision 		=> 'enable',
			snapresv    		=> '20%',
			thinminreserve 		=> '20%',
			thingrowthwarn 		=> '30%',
			thingrowthmax 		=> '40%',
			thinwarnsoftthres 	=> '50%',
			thinwarnhardthres 	=> '60%',
		}
	}

   You can create an init.pp file based on the above sample files and run the "puppet device" command , for example: 
   # puppet device

#-------------------------------------------------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------------------------------------------------	
