require 'puppet/provider/equallogic'

Puppet::Type.type(:equallogic_volume).provide(:equallogic_volume, :parent => Puppet::Provider::Equallogic) do
  @doc = 'Manage Equallogic Volume creation, modification and deletion.'

  confine :feature => :posix
  defaultfor :feature => :posix
  mk_resource_methods
  def flush
    Puppet.debug("Puppet::Provider::Equallogic_volume: Got to flush for resource #{@resource[:name]}.")
    # Check required resource state
    if @property_hash[:ensure] == :absent
      #Puppet.debug("Puppet::Provider::Equallogic_volume: Volume destroyed successfully.")
      return true
    end
    @property_hash.clear
  end

  def get_volume_info_for_vol
    ` python  "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeshow #{@resource[:name]}`
  end

  # Volume info getter
  def get_volinfo
    Puppet.debug("Puppet::Provider::Equallogic_volume get_volinfo: Getting volume info for volume #{@resource[:name]} of #{transport[:host]}.")
    response = get_volume_info_for_vol
    if response.match('Error logging')
      raise Puppet::Error, "Failed to retrieve volume information.#{response}"
    end
    volume_size = ''
    volume_name = ''
    volume_access = ''
    volume_template = ''
    volume_snap = ''
    volume_conn = ''
    volume_permission = ''
    volume_status = ''
    response.split("\n").each do |line|
      res = line.split(':')
      name = res[0].to_s.strip
      value = res[1].to_s.strip
      volume_status = get_status(name,value,volume_status)
      volume_size = get_size(name,value,volume_size)
      volume_name = get_name(name,value,volume_name)
      volume_access = get_access(name,value,volume_access)
      volume_template = get_template(name,value,volume_template)
      volume_snap = get_snap(name,value,volume_snap)
      volume_conn = get_conn(name,value,volume_conn)
      volume_permission = get_permission(name,value,volume_permission)

    end
    # Construct hash
    volume_info = { :name 	=> volume_name,
      :size 		=> volume_size,
      :snapshots	=> volume_snap,
      :status		=> volume_status,
      :permissions	=> volume_permission,
      :connections	=> volume_conn,
      :template	=> volume_template,
      :hostaccess	=> volume_access }
    Puppet.debug('Processed all volumes. Returning info array.')
    volume_info
  end

  def get_status(name,value,volume_status)
    if name == 'Status'
      volume_status = value
    end
    volume_status
  end

  def get_size(name,value,volume_size)
    if name == 'Size'
      volume_size = value
    end
    volume_size
  end

  def get_name(name,value,volume_name)
    if name == 'Name'
      volume_name = value
    end
    volume_name
  end

  def get_access(name,value,volume_access)
    if name == 'MultiHostAccess'
      volume_access = value
    end
    volume_access
  end

  def get_template(name,value,volume_template)
    if name == 'Template'
      volume_template = value
    end
    volume_template
  end

  def get_snap(name,value,volume_snap)
    if name == 'Snapshots'
      volume_snap = value
    end
    volume_snap
  end

  def get_conn(name,value,volume_conn)
    if name == 'Connections'
      volume_conn = value
    end
    volume_conn
  end

  def get_permission(name,value,volume_permission)
    if name == 'Permission'
      volume_permission = value
    end
    volume_permission
  end

  def create_volume_with_thinprovision
    `python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumecreate #{@resource[:name]}  #{@resource[:initsize]} #{@resource[:state]} #{@resource[:poolname]}  #{@resource[:snapreserve]}  #{@resource[:thinprovision]}  #{@resource[:thinminreserve]}  #{@resource[:thingrowthwarn]} #{@resource[:thingrowthmax]}  #{@resource[:thinwarnsoftthres]}  #{@resource[:thinwarnhardthres]} `
  end

  def create_volume_without_thinprovision
    `python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumecreate #{@resource[:name]}  #{@resource[:initsize]} #{@resource[:state]} #{@resource[:poolname]}  #{@resource[:snapreserve]}  #{@resource[:thinprovision]}  ""  "" ""  ""  "" `
  end

  def set_multihost_access
    `python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeshare #{@resource[:name]} #{@resource[:multihostaccess]}`
  end

  # Volume create.
  def create
    response = ''
    Puppet.debug("Puppet::Provider::Equallogic_volume: creating Equallogic Volume #{@resource[:name]} of initial size #{@resource[:initsize]}  with a state of #{@resource[:state]} in pool #{@resource[:poolname]} multiHostAccess = #{@resource[:multihostaccess]} thinminreserve =  #{@resource[:thinminreserve]} thingrowthwarn =  #{@resource[:thingrowthwarn]}. snapreserve =  #{@resource[:snapreserve]} thinprovision =  #{@resource[:thinprovision]} thingrowthmax =  #{@resource[:thingrowthmax]} thinwarnsoftthres =  #{@resource[:thinwarnsoftthres]}  thinwarnhardthres =  #{@resource[:thinwarnhardthres]}")

    ## check if the volume already exists or not
    if "#{@resource[:thinprovision]}" == 'enable'
      Puppet.info('Puppet::Provider::Equallogic_volume: Volume create call')
      response = create_volume_with_thinprovision
    else
      Puppet.info('Puppet::Provider::Equallogic_volume: Volume create call   - without thin provision')
      response = create_volume_without_thinprovision
    end
    #check if the volume has been created successfully
    if response.match('% Error') || response.match('Error: Only ASCII')
      raise Puppet::Error, "Failed to Create Volume  #{@resource[:name]} due the following: #{response}"
    else
      ## Add multihost access to volume as per user inputs
      response = set_multihost_access

      ##checking command execution
      if response.match('% Failed to share the volume')
        raise Puppet::Error, "Failed to #{@resource[:multihostaccess]} multi-host access on volume #{@resource[:name]}: #{response}"
      end
    end
    Puppet.info("Puppet::Provider::Equallogic_volume: Volume #{@resource[:name]} created successfully.")
    true
  end

  def set_volume_offline
    `python "#{$python_file_path}" #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeoffline #{@resource[:name]} `
  end

  def set_volume_online
    ` python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeonline #{@resource[:name]}`
  end

  def destroy_volume
    `python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumedelete #{@resource[:name]} `
  end

  def destroy
    Puppet.info("Puppet::Provider::Equallogic_volume: Destroying Equallogic Volume #{@resource[:name]}")
    response = set_volume_offline
    if response.match('% Error')
      raise Puppet::Error, "Failed to offline Volume  #{@resource[:name]} due the following: #{response}"
    end

    response = destroy_volume
    if response.match('% Error')
      raise Puppet::Error, "Failed to Delete Volume  #{@resource[:name]} due the following: #{response}"
    end
    Puppet.info("Puppet::Provider::Equallogic_volume:  Volume #{@resource[:name]} deleted successfully.")
    @property_hash[:ensure] = :absent
  end

  #getter method
  def state
    Puppet.debug('Inside state getter...')
    vol_state = nil
    volume_info = get_volinfo
    vol_name = volume_info[:name]
    if "#{@resource[:name]}" == vol_name
      Puppet.debug('Resource name matched...')
      vol_state =  volume_info[:status]
    end
    vol_state
  end

  def multihostaccess
    Puppet.debug('Inside multihostaccess getter...')
    vol_multihostaccess = nil
    volume_info = get_volinfo
    vol_name = volume_info[:name]
    if "#{@resource[:name]}" == vol_name
      Puppet.debug('Resource name matched...')
      vol_multihostaccess =  volume_info[:hostaccess].chomp('d')
    end
    vol_multihostaccess
  end

  #setter method
  def state=(value)
    Puppet.info("Puppet::Provider::Equallogic_volume: Setting Equalogic volume #{@resource[:name]} in #{value} state.")
    if "#{value}" == 'online'
      # Call python api for changing volume state
      response = set_volume_online
    else
      # Call python api for changing volume state
      response = set_volume_offline
    end
    if response.match('% Error')
      raise Puppet::Error, "Failed to set volume  #{@resource[:name]} in #{value} state: #{response}"
    end
  end

  def multihostaccess=(value)
    Puppet.info("Puppet::Provider::Equallogic_volume: #{value} Multi-host access on equalogic volume #{@resource[:name]}.")
    # Call python api for enable/disable multi-host access
    response = set_multihost_access
    if response.match('% Error')
      raise Puppet::Error, "Failed to enable multi-host access on volume #{@resource[:name]}: #{response}"
    end
  end

  def exists?
    volume_info = get_volinfo
    if "#{@resource[:ensure]}" == 'absent'
      vol_name = volume_info[:name]
      if "#{@resource[:name]}" == "#{vol_name}"
        @property_hash[:ensure] = :present
        return true
      end
      Puppet.info("Failed to Delete Volume #{@resource[:name]} does not exist.")
      @property_hash[:ensure] = :absent
      return false
    else
      vol_name = volume_info[:name]
      volume_state = volume_info[:status]
      volume_size = volume_info[:size]
      Puppet.debug(" Check volume in exists?  #{vol_name} :: #{volume_state} :: #{volume_size}")
      if "#{@resource[:name]}" == "#{vol_name}"
        Puppet.info("Puppet::Provider::Equallogic_volume: Volume #{@resource[:name]} already exists.")
        Puppet.info("Puppet::Provider::Equallogic_volume: Volume #{@resource[:name]} is of #{volume_size} and is in #{volume_state} state.")
        @property_hash[:ensure] = :present
        return true
      end
      @property_hash[:ensure] == :absent
    end
    @property_hash[:ensure] == :present
  end
end
