require 'puppet/provider/equallogic'

Puppet::Type.type(:equallogic_volume_access_chap).provide(:equallogic_volume_access_chap, :parent => Puppet::Provider::Equallogic) do
  @doc = 'Manage Equallogic Volume CHAP access creation and deletion.'

  confine :feature => :posix
  defaultfor :feature => :posix
  $chap_user=''
  $id=''
  mk_resource_methods

  def flush
    Puppet.debug("Puppet::Provider::Equallogic_volume_access_chap Got to flush for resource #{@resource[:name]}.")
    # Check required resource state
    if @property_hash[:ensure] == :absent
      return true
    end
    @property_hash.clear
  end

  #
  def get_all_vol
    `"#{$volume_file_path}" show `
  end   
  
  # Volume info getter
  def self.get_volinfo
    Puppet.debug('Puppet::Provider::Equallogic_volume get_volinfo: getting volume info for all volumes.')

    # Pull back current volume-size.
    response = get_all_vol
    Puppet.debug("Puppet::Provider::Equallogic_volume_access_chap Volume show response :: #{response}")
    # Update other attributes after resource creation.
    Puppet.debug('Puppet::Provider::Equallogic_volume get_volinfo: Pulling back volumes array.')
    volume_info = []
    counter = 0
    volcount = 1
    response.split("\n").each do |line|
      if counter < 3
        counter += 1
        next
      else
        # Pull out relevant info and add to array
        volume_info << get_vol_info(line)
        volcount += 1
      end
    end
    Puppet.debug('Processed all volumes. Returning info array.')
    # Return volume_info array
    volume_info
  end

  def get_vol_info(line)
    array = line.split(' ')
    vol_name = array[0]
    Puppet.debug("Puppet::Provider::Equallogic_volume_clone get_volinfo: Processing volume #{vol_name}.")
    # Construct hash
    vol_info = { :name => vol_name }
    vol_info
  end

  def create_volume_chap_access(chap_user_name)
    ` python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccessassociatechapuser #{@resource[:name]} "#{chap_user_name}"`
  end
    
  def create
    Puppet.info('Puppet::Provider::Equallogic_volume: Volume access create call')
    chap_user_list = @resource[:chap_user_name]
    chap_users = chap_user_list.split(',')
    chap_users.each do |chap_user_name|
      response = create_volume_chap_access(chap_user_name)
      if response.match('% Error')
        raise Puppet::Error, "Unable to create the chap access to the volume #{@resource[:name]} because of the following exception: #{response}."
      end
      Puppet.info('Puppet::Provider::Equallogic_volume: Volume access created successfully.')
      Puppet.debug("Puppet::Provider::Equallogic_volume: Volume access created response :: #{response}")
    end
    true
  end

  def exists?
    Puppet.info("Puppet::Provider::Equallogic_volume_access_chap checking existence of volume ...")
    Puppet.debug(" resource[:ensure]  ==  #{@resource[:ensure]}")
    chap_user_list = @resource[:chap_user_name]
    chap_users = chap_user_list.split(',')
    if "#{@resource[:ensure]}" == 'absent'
      chap_users_list = check_chap_users(chap_users)
      if chap_users_list.size > 0
        Puppet.info('Chap access does not  exists at volume')
        false
      else
        true
      end
    else
      chap_users_list = check_chap_users(chap_users)
      if chap_users_list.size == 0
        Puppet.info('Chap access already exists at volume')
        true
      else
        false
      end
    end
  end

  def show_volume_access
    ` python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccessshow #{@resource[:name]}`
  end   
  
  def check_chap_users(chap_user_list)
    response = show_volume_access
    if response.match('% Error')
      raise Puppet::Error, "Unable to retrieve the chap access for the volume #{@resource[:name]} because of the following exception: #{response}."
    end
    counter = 0
    response.split("\n").each do |line|
      if counter < 3
        counter += 1
        next
      end
      if line.to_s.strip.length == 0
        next
      end
      res = line.to_s.strip.split(' ')
      tempid = res[0]
      temp_user_name = res[3]
      if line.to_s.strip.length == 0
        next
      end

      if tempid.to_s.strip =~ /^\d+$/ && !(line.strip.match('iqn'))
        $id = "#{tempid}"
        $chap_user = "#{temp_user_name}"
      else

        line = line.to_s.strip
        $chap_user = "#{$chap_user}#{line}"

      end
      if chap_user_list.include?($chap_user)
        chap_user_list.delete($chap_user)
      end
    end
    return chap_user_list
  end

  def destroy
    Puppet.info('Puppet::Provider::Equallogic_volume: Access delete Equallogic Volume')
    response = show_volume_access
    if response.match('Error logging')
      raise Puppet::Error, "Unable to retrieve the chap access for the volume#{@resource[:name]} due the following: #{response} . Check username/password/ipaddress."
    end
    if response.match('% Error')
      raise Puppet::Error, "Unable to retrieve the chap access for the volume #{@resource[:name]} because of the following exception: #{response}."
    end
    access_id_list = Array.new
    counter = 0
    chap_user_list = @resource[:chap_user_name]
    chap_users = chap_user_list.split(',')
    response.split("\n").each do |line|
      if counter < 3
        counter += 1
        next
      end
      if line.to_s.strip.length == 0
        next
      end
      access_id_list = populate_accessid_list(line,chap_users,access_id_list)
    end
    delete_accessid_list(access_id_list)
    @property_hash[:ensure] = :absent
  end

  def populate_accessid_list(line,chap_users,access_id_list)
    res = line.to_s.strip.split(' ')
    tempid = res[0]
    iq = res[1]
    temp_user_name = res[3]
    tempip = res[2]
    if line.to_s.strip.length == 0
      return ''
    end

    if tempid.to_s.strip =~ /^\d+$/ && !(line.strip.match('iqn'))
      iqnum = "#{iq}"
      $id = "#{tempid}"
      ip = "#{tempip}"
      $chap_user = "#{temp_user_name}"
    else

      line = line.to_s.strip
      iqnum = "#{iqnum}#{line}"
      $chap_user = "#{$chap_user}#{line}"

    end
    if chap_users.include?($chap_user)
      access_id_list.push($id)
    end
    return access_id_list
  end

  def delete_vol_access(accessId)
    ` python #{$python_file_path}  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccessdelete #{@resource[:name]}  "#{accessId}"`
  end
    
  def delete_accessid_list(access_id_list)
    Puppet.debug('Inside delete_accessid_list........')

    access_id_list.each do |accessId|
      response = delete_vol_access(accessId)
      if response.match('% Error')
        raise Puppet::Error, "Unable to delete the chap access to/for the volume #{@resource[:name]} because of the following exception: # {response}."
      end
    end
  end

end

