require 'puppet/provider/equallogic'

Puppet::Type.type(:equallogic_volume_access_iqn).provide(:equallogic_volume_access_iqn, :parent => Puppet::Provider::Equallogic) do
  @doc = "Manage Equallogic Volume IQN access creation and deletion."

  confine :feature => :posix
  defaultfor :feature => :posix
  $id=""
  $ip=""
  $iqnum=""
  mk_resource_methods

  def flush
    Puppet.debug("Puppet::Provider::Equallogic_volume: Got to flush for resource #{@resource[:name]}.")
    # Check required resource state
    if @property_hash[:ensure] == :absent
      return true
    end
    @property_hash.clear
  end

  #
  def get_all_vol
    `"#{$volume_file_path}" show `
  end

  # Volume info getter
  def self.get_volinfo
    Puppet.debug('Puppet::Provider::Equallogic_volume get_volinfo: getting volume info for all volumes.')
    # Pull back current volume-size.
    response = get_all_vol
    volume_info = []
    counter = 0
    volcount = 1
    response.split("\n").each do |line|
      if counter < 3
        counter = counter + 1
        next
      else
        volume_info << create_vol_info(line)
        volcount += 1
      end
    end
    Puppet.debug('Processed all volumes. Returning info array.')
    # Return volume_info array
    volume_info
  end

  def create_vol_info(line)
    array = line.split(' ')
    vol_name = array[0]
    Puppet.debug("Puppet::Provider::Equallogic_volume get_volinfo: Processing volume #{vol_name}.")
    vol_size = array[2]
    vol_status = array[4]
    vol_snap = array[3]
    vol_permissions = array[5]
    vol_connections = array[6]
    vol_template = array[7]
    Puppet.debug("Puppet::Provider::Equallogic_volume get_volinfo: Vol_name = #{vol_name}, vol_size_bytes = #{vol_size}, vol_state = #{vol_status}, vol_snapshot = #{vol_snap}, vol_permissions = #{vol_permissions}, vol_connections = #{vol_connections}.")
    # Construct hash
    { :name         => vol_name,
      :size         => vol_size,
      :snapshots    => vol_snap,
      :status       => vol_status,
      :permissions  => vol_permissions,
      :connections  => vol_connections,
      :template     => vol_template
    }
  end

  def create_vol_iqn_access(iqnOrIp)
    ` python #{$python_file_path} #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccesscreate #{@resource[:name]}  "#{iqnOrIp}"`
  end

  def create
    Puppet.info('Puppet::Provider::Equallogic_volume: Volume access create call')
    iqn_list = @resource[:iqnorip]
    if iqn_list.is_a?(String)
	iqn_list = iqn_list.split(',')
    end
    iqn_list.each do |iqnOrIp|
      response = create_vol_iqn_access(iqnOrIp)
      if response.match('% Error')
        raise Puppet::Error, "Unable to create the iqn access to the volume #{@resource[:name]} because of the following exception: #{response}."
      end
      Puppet.info('Puppet::Provider::Equallogic_volume: Volume access created successfully.')
      Puppet.debug("Puppet::Provider::Equallogic_volume: Volume access created response :: #{response}")
    end
    true
  end

  def show_volume_access
    ` python "#{$python_file_path}"  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccessshow #{@resource[:name]}`
  end

  def destroy
    Puppet.info('Puppet::Provider::Equallogic_volume: access delete Equallogic Volume')
    response = show_volume_access
    if response.match('Error logging')
      raise Puppet::Error, "Unable to retrieve the chap access for the volume#{@resource[:name]} due the following: #{response} . Check username/password/ipaddress."
    end
    if response.match('% Error')
      raise Puppet::Error, "Unable to retrieve the iqn access for the volume #{@resource[:name]} because of the following exception: #{response}."
    end
    accessIdList = Array.new
    counter = 0
    iqn_list = @resource[:iqnorip].dup
     if iqn_list.is_a?(String)
        iqn_list = iqn_list.split(",")
    end
    response.split("\n").each do |line|
      if counter < 3
        counter += 1
        next
      end
      if line.to_s.strip.length == 0
        next
      end
      accessIdList = populate_accessid_list(line,iqn_list,accessIdList)
    end
    delete_accessid_list(accessIdList)
    @property_hash[:ensure] = :absent
  end

  def populate_accessid_list(line,iqns,accessIdList)
    res = line.to_s.strip.split(" ")
    tempid = res[0]
    iq = res[1]
    tempip = res[2]
    if tempid.to_s.strip =~ /^\d+$/
      $iqnum = "#{iq}"
      $id = "#{tempid}"
      $ip = "#{tempip}"
    else
      line = line.to_s.strip
      $iqnum = "#{$iqnum}#{line}"
    end
    if iqns.include?($iqnum)
      accessIdList.push($id)
    elsif iqns.include?($ip)
      accessIdList.push($id)
    end
    accessIdList
  end

  def delete_vol_access(accessId)
    ` python #{$python_file_path}  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} volumeaccessdelete #{@resource[:name]}  "#{accessId}"`
  end

  def delete_accessid_list(accessIdList)
    accessIdList.each do |accessId|
      response = delete_vol_access(accessId)
      if response.match('% Error')
        raise Puppet::Error, "Unable to delete the iqn access to/for the volume #{@resource[:name]} because of the following exception: # {response}."
      end
    end
  end

  def exists?
    Puppet.debug("Puppet::Provider::Equallogic_volume: checking existance of Equallogic Volume #{@resource[:name]}")
    Puppet.debug(" resource[:ensure]  ==  #{@resource[:ensure]}")
    iqn_list = @resource[:iqnorip]
    @input_iqn = @resource[:iqnorip].dup
     if iqn_list.is_a?(String)
        iqn_list = iqn_list.split(',')
    end
    if "#{@resource[:ensure]}" == 'absent'
      iqnlist = checkiqn(iqn_list)
      if iqnlist.size != iqn_list.size
        Puppet.info('QN access does not exists at volume')
        return false
      else
        return true
      end
    else
      iqnlist = checkiqn(iqn_list)
      if iqnlist.size == 0
        Puppet.info('IQN access already exists at volume')
        return true
      else
        return false
      end
    end
  end

  def checkiqn(iqnList)
    response = show_volume_access
    if response.match('% Error')
      raise Puppet::Error, "Unable to retrieve the iqn access for the volume #{@resource[:name]} because of the following exception: #{response}."
    end
    counter = 0
    response.split("\n").each do |line|
      if counter < 3
        counter += 1
        next
      end
      if line.to_s.strip.length == 0
        next
      end
      res = line.to_s.strip.split(' ')
      tempid = res[0]
      iq = res[1]
      tempip = res[2]
      if tempid.to_s.strip =~ /^\d+$/
        $iqnum = "#{iq}"
        $id = "#{tempid}"
        $ip = "#{tempip}"
      else
        line = line.to_s.strip
        $iqnum = "#{$iqnum}#{line}"
      end
      if iqnList.include?($iqnum)
        iqnList.delete($iqnum)
      elsif iqnList.include?($ip)
        iqnList.delete($ip)
      end
    end
    return iqnList
  end

end

