require 'puppet/provider/equallogic'
Puppet::Type.type(:equallogic_volume_chap_user).provide(:equallogic_volume_chap_user, :parent => Puppet::Provider::Equallogic) do
  @doc = "Manage Equallogic CHAP user creation and deletion"

  confine :feature => :posix
  defaultfor :feature => :posix
  mk_resource_methods

  def flush
    Puppet.debug("Puppet::Provider::Equallogic_volume_chap_user: Got to flush for resource #{@resource[:name]}.")
    # Check required resource state
    if @property_hash[:ensure] == :absent
      return true
    end
    @property_hash.clear
  end

  # Get chap user info. Output will look like:
  #
  # Name                           Password         Status     Admin Account
  # ------------------------------ ---------------- ---------- ----------------
  # rutland01                      password          enable     grpadmin
  # cgully                         password          enable     grpadmin
  # gs1user1                       password          enable     grpadmin
  def get_chap_user_list
    `python #{$python_file_path}  #{transport[:host]}  #{transport[:user]}  #{transport[:password]} "show chapuser"`
  end  

  # Volume info getter
  def get_chap_user_info
    Puppet.debug('Puppet::Provider::Equallogic_volume_chap_user get_volinfo: getting volume info for all volumes.')

    # Pull back current chap user list.
    response = get_chap_user_list
    if response.match('Error')
      raise Puppet::Error, "Failed to get chap user  #{@resource[:name]} info  due the following: #{response}"
    end
    # Update other attributes after resource creation.
    Puppet.debug('Puppet::Provider::Equallogic_volume_chap_user get_chap_user_info: Pulling back chap user array.')
    chap_user_info = []
    counter = 0
    volcount = 1
    response.split("\n").each do |line|
      if counter < 2
        # Skip header lines
        counter += 1
        next
      else
        # Pull out relevant info
        array = line.split(' ')
        chap_user_name = array[0]
        chap_password = array[1]
        Puppet.debug("Puppet::Provider::Equallogic_volume_chap_user get_chap_user_info: Processing chap user #{chap_user_name}.")
        # Construct hash
        chap_info = { :name => chap_user_name, :chap_password => chap_password}
        # Add to array
        chap_user_info << chap_info
        volcount = volcount + 1
      end
    end
    Puppet.debug('Processed all chap users. Returning info array.')
    # Return chap user info array
    chap_user_info
  end

  def create_chap_user
    decrypted_password = decrypt_password(resource[:chap_password])
    ` python #{$python_file_path} #{transport[:host]}  #{transport[:user]}  #{transport[:password]} chapusercreate #{@resource[:name]}  #{decrypted_password} #{@resource[:state]}`
  end

  def decrypt_password(pass)
    if resource.decrypt?
      # this requires that asm-deployer is symlinked into the modulepath
      require 'asm/cipher'
      ASM::Cipher.decrypt_string(pass)
    else
      pass
    end
  end
    
  def create
    Puppet.info('Puppet::Provider::Equallogic_volume_chap_user: chap user create call')
    response = create_chap_user
    if response.match('% Error')
      raise Puppet::Error, "Unable to create the chap user #{@resource[:name]} because of the following exception: #{response}."
    end
    Puppet.info('Puppet::Provider::Equallogic_volume_chap_user: chap user created successfully.')
    Puppet.debug("Puppet::Provider::Equallogic_volume_chap_user: chap user creation response :: #{response}")
    return true
  end

  def delete_chap_user
    ` python #{$python_file_path} #{transport[:host]}  #{transport[:user]}  #{transport[:password]} chapuserdelete  #{@resource[:name]} `
  end
    
  def destroy
    Puppet.debug('Puppet::Provider::Equallogic_volume_chap_user: delete chap user')
    response = delete_chap_user
    if response.match('% Error')
      raise Puppet::Error, "Unable to delete the chap user #{@resource[:name]} because of the following exception: #{response}."
    end
    @property_hash[:ensure] = :absent

  end

  def exists?
    Puppet.info("Puppet::Provider::Equallogic_volume_chap_user_create_chap_user: Checking existence of Equallogic Volume #{@resource[:name]}")
    chap_info = get_chap_user_info
    Puppet.debug('Iterating chap_info')
    chap_info.each do |chap_user|
      # Resource existence validation
      Puppet.debug('Puppet::Provider::Equallogic_volume_chap_userr self.instances: Comparing if resource already exists at equallogic...')
      if "#{@resource[:name]}" == chap_user[:name]
        Puppet.info("Puppet::Provider::Equallogic_volume_chap_user_ self.instances: Resource name #{@resource[:name]} already exists.")
        unless chap_user[:chap_password] == decrypt_password(resource[:chap_password])
          Puppet.info("Puppet::Provider::Equallogic_volume_chap_user get_chap_user_info: Provided password does not match existing password for chap user #{@resource[:name]}")
          raise Puppet::Error, "Provided password does not match existing password for chap user #{@resource[:name]}"
        end
        return true
      end
    end
    false
  end

end
