Puppet::Type.newtype(:equallogic_volume) do
  @doc = 'Manage Equallogic Volume creation, modification and deletion.'

  ensurable

  newparam(:name) do
    desc 'The volume name. Valid characters are a-z, 1-9 & hyphen.'
    isnamevar
    validate do |value|
      raise ArgumentError, '%s is not a valid volume name.' % value unless value =~ /^[\w-]{1,63}$/
      raise ArgumentError, '%s is not a valid volume name.Underscore not accepted.' % value  unless !(value.to_s.include? '_')
    end
  end

  newproperty(:state) do
    desc 'The volume state. Valid options are: online, offline.'
    newvalues(:online, :offline)
    defaultto :online
  end

  newparam(:initsize) do
    desc 'The initial volume size. Valid format is 1-9(mgt).'
    defaultto '1g'
    
    validate do |value|
      unless value =~ /^\d+([mgt]$|(MB|GB|TB)$)/
       	raise ArgumentError, '%s is not a valid initial volume size.' % value
      end
    end
    
     munge do |value|
      if value =~ /^\d+(KB)$/
        value.sub!(/(KB)/,'k')
      elsif value =~ /^\d+(MB)$/
        value.sub!(/(MB)/,'m')
      elsif value =~ /^\d+(GB)$/
        value.sub!(/(GB)/,'g')
      elsif value =~ /^\d+(TB)$/
        value.sub!(/(TB)/,'t')
      end
    end
  
  end

  newparam(:thinprovision) do
    desc 'The language code this volume should use.'
    defaultto 'enable'
    newvalues(:enable, :disable)
  end

  newparam(:snapreserve) do
    desc 'The percentage of space to reserve for snapshots.'
    isrequired

    validate do |value|
      raise ArgumentError, '%s is not a valid snapreserve.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end

  newparam(:thinminreserve) do
    desc 'The percentage of minimum space to reserve .'
    isrequired
    validate do |value|
      raise ArgumentError, '%s is not a valid thinminreserve.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end

  newparam(:thingrowthwarn) do
    desc 'The percentage of growth warning .'
    isrequired

    validate do |value|
      raise ArgumentError, '%s is not a valid thingrowthwarn.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end

  newparam(:thingrowthmax) do
    desc 'The percentage of maximum growth warning .'
    isrequired

    validate do |value|
      raise ArgumentError, '%s is not a valid thingrowthmax.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end

  newparam(:thinwarnsoftthres) do
    desc 'The percentage of soft threshold  warning .'
    isrequired

    validate do |value|
      raise ArgumentError, '%s is not a valid thinwarnsoftthres.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end

  newparam(:thinwarnhardthres) do
    desc 'The percentage of hard threshold warning .'
    isrequired

    validate do |value|
      raise ArgumentError, '%s is not a valid thinwarnhardthres.' % value unless value =~ /^\d+[%]$/
      raise ArgumentError, 'Puppet::Type::Equallogic_volume: Reserved percentage must be between 0 and 100.' unless value.to_i.between?(0,100)
    end
  end
  
  newparam(:poolname) do
    desc 'The pool name where new volume will be created .'
    defaultto 'default'
    validate do |value|
        unless value =~ /^[a-zA-Z0-9\-:]{1,63}$/
          raise ArgumentError, 'The %s is an invalid pool name.' % value
        end
      end
    end
  
  newproperty(:multihostaccess) do
    desc 'MultiHostAccess volume access. Valid options are: enable and disable .'
    newvalues(:enable, :disable)
    defaultto :disable
  end

end
