Puppet::Type.newtype(:equallogic_volume_access_iqn) do
  @doc = "Manage Equallogic Volume access iqn."
  ensurable
  newparam(:name) do
    desc "The volume name. Valid characters are a-z, 1-9 & hyphen."
    isnamevar
    validate do |value|
      unless value =~ /^[a-zA-Z0-9-]{1,63}$/
        raise ArgumentError, "The %s is an invalid volume resource name." % value
      end
    end
  end

  newparam(:iqnorip) do
    desc "The valid iqn or ip address."
    validate do |value|
      if(value.is_a?(String))
        value = value.split(",")
      end
      value.each do |val|
        if (val.match("iqn"))
          unless val =~ /^iqn.(\d+)-(\d+).(\S+)$/
            raise ArgumentError, "%s is not a valid iqn." % val
          end
        else
          val = val.gsub(/\*/, "0")
          unless val =~ Resolv::IPv4::Regex
            raise ArgumentError, "%s is not a valid IP address." % val
          end
        end
      end
    end
  end

end
