require 'puppet/parameter/boolean'

Puppet::Type.newtype(:equallogic_volume_chap_user) do
  @doc = "Equallogic chap user creation and deletion."
  ensurable
  newparam(:name) do
    desc "The chap user name. Valid characters are a-z, 1-9, hyphen & underscore."
    isnamevar
    validate do |value|
      if value.strip.length == 0
        raise ArgumentError, "The chap user name cannot be empty." % value
      end
      unless value =~ /^[\w+ -]{1,63}$/
        raise ArgumentError, "The %s is an invalid chap user name." % value
      end
    end
  end

  newparam(:chap_password) do
    desc "The chap password . Valid characters are a-z, 1-9 & underscore."
    validate do |value|
      unless value =~ /^[\S]{0,254}$/
        raise ArgumentError, "The %s is an invalid chap password." % value
      end
    end
  end

  newparam(:state) do
    desc "The chap user state. Valid characters are a-z, 1-9,- & underscore."
    defaultto "enable"
    newvalues(:enable, :disable)
  end

  newparam(:decrypt, :boolean => true, :parent => Puppet::Parameter::Boolean) do
    desc 'Whether the chap password is in ASM\'s encrypted format'
    defaultto false
  end

end

