#
# == Define: equallogic::create_vol_chap_user_access
#
#
# Utility for creating volumes in equallogic storage.
#
# Parameters:
#
# Actions:
#
# Requires:
#
# Sample Usage:
#   equallogic::create_vol_chap_user_access  {'testvolume' :
#     size => '1g' ,
#     chap_user_name => 'chapUser' ,
#     passwd => 'passwd',
#   }
#
#
define equallogic::create_vol_chap_user_access(
  $size,
  $thinprovision = "enable",
  $snapreserve = "100%",
  $thinminreserve = "10%",
  $thingrowthwarn = "60%",
  $thingrowthmax = "100%",
  $thinwarnsoftthres = "60%",
  $thinwarnhardthres = "90%",
  $multihostaccess = "enable",
  $auth_type ,
  $chap_user_name = "",
  $passwd = "",
  $iqnorip = "",
  $decrypt = false,
  $poolname = "default",
) {

  equallogic_volume {"$name":
    state => "online",
    initsize => "$size",
    thinprovision => "$thinprovision",
    snapreserve => "$snapreserve",
    thinminreserve => "$thinminreserve",
    thingrowthwarn => "$thingrowthwarn",
    thingrowthmax => "$thingrowthmax",
    thinwarnsoftthres => "$thinwarnsoftthres",
    thinwarnhardthres => "$thinwarnhardthres",
    multihostaccess => "$multihostaccess",
    poolname => "$poolname",
    ensure => "present",
  }

  if ( $auth_type == "chap" ) {
    if empty($chap_user_name) == false {
      equallogic_volume_chap_user {"$chap_user_name":
        chap_password => "$passwd",
        decrypt       => $decrypt,
        state         => "enable",
        ensure        => "present",
        require       =>  Equallogic_volume["$name"],
      }

      equallogic_volume_access_chap {"$name":
        chap_user_name => "$chap_user_name",
        ensure         => "present",
        require        => Equallogic_volume_chap_user["$chap_user_name"],
      }
    }
  }

  if ( $auth_type == "iqnip" ) {
    if empty($iqnorip) == false {
      equallogic_volume_access_iqn { "$name":
        ensure  => "present",
        iqnorip => $iqnorip,
        require =>  Equallogic_volume[$name],
      }
    }
  }
}

