#
# == Define: equallogic::create_vol_iqnorip_access
#
#
# Utility for creating volumes in equallogic storage.
#
# Parameters:
#
# Actions:
#
# Requires:
#
# Sample Usage:
#   equallogic::create_vol_iqnorip_access  {'testvolume-iqnorip' :
#   iqnorip => ['10.10.10.10'],
#   size => '1g',
# }
#
#
define equallogic::create_vol_iqnorip_access (
  $iqnorip = '',
  $size,
  $thinprovision = "enable",
  $snapreserve = "100%",
  $thinminreserve = "10%",
  $thingrowthwarn = "60%",
  $thingrowthmax = "100%",
  $thinwarnsoftthres = "60%",
  $thinwarnhardthres = "90%",
  $multihostaccess = "enable",
  $decrypt = false,
  $poolname = "default",
) {

  $iqn = strip(split($iqnorip, ','))

  equallogic_volume { $name :
    ensure            => "present",
    state             => "online",
    initsize          => $size,
    thinprovision     => $thinprovision,
    snapreserve       => $snapreserve,
    thinminreserve    => $thinminreserve,
    thingrowthwarn    => $thingrowthwarn,
    thingrowthmax     => $thingrowthmax,
    thinwarnsoftthres => $thinwarnsoftthres,
    thinwarnhardthres => $thinwarnhardthres,
    multihostaccess   => $multihostaccess,
    poolname => "$poolname",
  }
  
  if empty($iqn) == false {
    equallogic_volume_access_iqn { "$name":
      ensure  => "present",
      iqnorip => $iqn,
      require =>  Equallogic_volume[$name],
    }
  }
  
}

