require 'spec_helper'
require 'rspec/expectations'

describe Puppet::Type.type(:equallogic_volume_access_chap).provider(:equallogic_volume_access_chap) do
  let(:resource) {Puppet::Type.type(:equallogic_volume_access_chap).new(
                                                                       {
                                                                           :name           => 'TestVol',
                                                                           :chap_user_name => 'TestUser',
                                                                           :ensure         => 'present',
                                                                           :require        => "Equallogic_volume_chap_user['TestChapUser']"
                                                                       }
  )}
  let(:provider) {resource.provider}

  before :each do
    @fixture_dir = File.join(Dir.pwd,'spec','fixtures')
    Puppet::Equallogic::Util.stub(:get_transport).and_return({
                                                                 :host => '172.17.4.10',
                                                                 :user => 'grpadmin',
                                                                 :password => 'password'
                                                             })
  end

  describe 'exists?' do
    context 'when user does not exist' do
      it 'returns false' do
        provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show')))
        expect(provider.exists?).to be false
      end
    end
    context 'when user already exists' do
      it 'returns true' do
        provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show_present')))
        expect(provider.exists?).to be true
      end
    end
  end

  describe 'create' do
    it 'creates the user access' do
      provider.stub(:create_volume_chap_access).and_return("\nAccess control record created with ID 3.\n")
      expect(provider.create).to be true
    end
  end

  describe 'destroy' do
    it 'removes the user access' do
      provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show_present')))
      provider.stub(:delete_vol_access).and_return("Access control record deletion succeeded.\n")
      provider.destroy.should == :absent
    end
  end

end