require 'spec_helper'
require 'rspec/expectations'

describe Puppet::Type.type(:equallogic_volume_access_iqn).provider(:equallogic_volume_access_iqn) do
  let(:resource) {Puppet::Type.type(:equallogic_volume_access_iqn).new(
      {
          :name     => 'TestVol',
          :ensure   => 'present',
          :iqnorip  => ['0.0.0.0'],
          :require  => 'Equallogic_volume[TestVol]'
      }
  )}
  let(:provider) {resource.provider}

  before :each do
    @fixture_dir = File.join(Dir.pwd,'spec','fixtures')
    Puppet::Equallogic::Util.stub(:get_transport).and_return({
                                                                 :host => '172.17.4.254',
                                                                 :user => 'grpadmin',
                                                                 :password => 'password'
                                                             })
  end

  describe 'exists?' do
    context 'when iqn is already created' do
      it 'returns true' do

      end
    end

    context 'when iqn is not created' do
      it 'returns false' do

      end
    end
  end

  describe 'checkiqn' do
    context 'when iqn exists' do
      it 'returns iqn list' do
        provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show_iqn')))
        provider.checkiqn(['0.0.0.0']).should == []
      end
    end
    context 'when iqn does not exist' do
      it 'returns something' do
        provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show')))
        provider.checkiqn(['0.0.0.0']).should == ['0.0.0.0']
      end
    end
  end

  describe 'create' do
    it 'creates the iqn' do
      provider.stub(:create_vol_iqn_access).and_return("Access control record created with ID 3.\n")
      expect(provider.create).to be true
    end
  end

  describe 'destroy' do
    it 'removes the iqn' do
      provider.stub(:show_volume_access).and_return(File.read(File.join(@fixture_dir,'volume_access_show_iqn')))
      provider.stub(:delete_vol_access).and_return("Access control record deletion succeeded.\n")
      provider.destroy.should == :absent
    end
  end

end